﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Reflection;
using System.Collections;

namespace System.ServiceModel
{
    public static class DynamicServiceHelper
    {
        static object _syncRoot = new object();
        static FieldInfo hostingManagerField;
        static MethodInfo ensureInitialized;
        static FieldInfo serviceActivationsField;

        static DynamicServiceHelper()
        {
            ensureInitialized = typeof(ServiceHostingEnvironment).GetMethod("EnsureInitialized", BindingFlags.Static | BindingFlags.NonPublic | BindingFlags.InvokeMethod);
            hostingManagerField = typeof(ServiceHostingEnvironment).GetField("hostingManager", BindingFlags.Static | BindingFlags.NonPublic | BindingFlags.GetField);
        }

        public static void EnsureInitialized()
        {
            ensureInitialized.Invoke(null, new object[] { });
        }

        public static void RegisterService(string addr, Type factory, Type service)
        {

            // This is the code that injects the service activation configuration.
            // In WCF 4 we have the "not very well known" CBA Services (Configuration Based Activation)
            // to allow us to define "file-less" WCF Services (like our Service2 here in this demo) where
            // we have a section in our web.config to register services without the need of having a physical .svc file.
            // The section in the web.config looks like this:
            //
            //      <serviceHostingEnvironment multipleSiteBindingsEnabled="true" >
            //          <serviceActivations>
            //              <add relativeAddress="Service2.svc" service="WcfService2.Service2" />
            //          </serviceActivations>
            //      </serviceHostingEnvironment>
            //
            // And is this configuration what we are going to inject at runtime to simulate having that 
            // information in our web.config, while we haven't.

            lock (_syncRoot)
            {
                object hostingManager = hostingManagerField.GetValue(null);
                
                if (serviceActivationsField == null)
                {
                    serviceActivationsField = hostingManager.GetType().GetField("serviceActivations", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.GetField);
                }
                
                Hashtable serviceActivations = (Hashtable)serviceActivationsField.GetValue(hostingManager);

                string value = string.Format("{0}|{1}|{2}", addr, factory.AssemblyQualifiedName, service.AssemblyQualifiedName);

                if (!serviceActivations.ContainsKey(addr))
                {
                    serviceActivations.Add(addr, value);
                }
            }
        }
    }
}